@@ -1,6 +1,6 @@
 /********************************************
 bi_funct.c
-copyright 2008-2016,2020, Thomas E. Dickey
+copyright 2008-2021,2023, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.c,v 1.111 2020/01/06 10:01:20 tom Exp $
+ * $MawkId: bi_funct.c,v 1.118 2023/07/24 08:23:33 tom Exp $
  */
 
 #include <mawk.h>
@@ -54,7 +54,6 @@ the GNU General Public License, version 2, 1991.
 const BI_REC bi_funct[] =
 {				/* info to load builtins */
 
-   { "length",   bi_length,   0, 1 },	/* special must come first */
    { "index",    bi_index,    2, 2 },
    { "substr",   bi_substr,   2, 3 },
    { "sprintf",  bi_sprintf,  1, 255 },
@@ -93,12 +92,7 @@ bi_funct_init(void)
     register const BI_REC *p;
     register SYMTAB *stp;
 
-    /* length is special (posix bozo) */
-    stp = insert(bi_funct->name);
-    stp->type = ST_LENGTH;
-    stp->stval.bip = bi_funct;
-
-    for (p = bi_funct + 1; p->name; p++) {
+    for (p = bi_funct; p->name; p++) {
 	stp = insert(p->name);
 	stp->type = ST_BUILTIN;
 	stp->stval.bip = p;
@@ -128,11 +122,6 @@ bi_length(CELL *sp)
 
     TRACE_FUNC("bi_length", sp);
 
-    if (sp->type == 0)
-	cellcpy(sp, field);
-    else
-	sp--;
-
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
     len = string(sp)->len;
@@ -144,6 +133,18 @@ bi_length(CELL *sp)
     return_CELL("bi_length", sp);
 }
 
+/* length (size) of an array */
+CELL *
+bi_alength(CELL *sp)
+{
+    TRACE_FUNC("bi_alength", sp);
+
+    sp->type = C_DOUBLE;
+    sp->dval = (double) ((ARRAY) sp->ptr)->size;
+
+    return_CELL("bi_alength", sp);
+}
+
 char *
 str_str(char *target, size_t target_len, const char *key, size_t key_len)
 {
@@ -193,7 +194,6 @@ bi_index(CELL *sp)
 {
     size_t idx;
     size_t len;
-    const char *p;
 
     TRACE_FUNC("bi_index", sp);
 
@@ -202,6 +202,7 @@ bi_index(CELL *sp)
 	cast2_to_s(sp);
 
     if ((len = string(sp + 1)->len)) {
+	const char *p;
 	idx = (size_t) ((p = str_str(string(sp)->str,
 				     string(sp)->len,
 				     string(sp + 1)->str,
@@ -451,7 +452,7 @@ bi_mktime(CELL *sp)
 	my_tm.tm_mon -= 1;
 	result = mktime(&my_tm);
     }
-    TRACE(("...bi_mktime(%s) ->%s", sval->str, ctime(&result)));
+    TRACE(("...bi_mktime(%s) ->%s", sval ? sval->str : "?", ctime(&result)));
 
     cell_destroy(sp);
     sp->type = C_DOUBLE;
@@ -476,7 +477,6 @@ bi_strftime(CELL *sp)
     int n_args;
     int utc;
     STRING *sval = 0;		/* strftime(sval->str, timestamp, utc) */
-    char buff[128];
     size_t result;
 
     TRACE_FUNC("bi_strftime", sp);
@@ -517,7 +517,10 @@ bi_strftime(CELL *sp)
     else
 	ptm = localtime(&rawtime);
 
-    result = strftime(buff, sizeof(buff) / sizeof(buff[0]), format, ptm);
+    result = strftime(sprintf_buff,
+		      (size_t) (sprintf_limit - sprintf_buff),
+		      format,
+		      ptm);
     TRACE(("...bi_strftime (%s, \"%d.%d.%d %d.%d.%d %d\", %d) ->%s\n",
 	   format,
 	   ptm->tm_year,
@@ -528,12 +531,12 @@ bi_strftime(CELL *sp)
 	   ptm->tm_sec,
 	   ptm->tm_isdst,
 	   utc,
-	   buff));
+	   sprintf_buff));
 
     if (sval)
 	free_STRING(sval);
 
-    sp->ptr = (PTR) new_STRING1(buff, result);
+    sp->ptr = (PTR) new_STRING1(sprintf_buff, result);
 
     while (n_args > 1) {
 	n_args--;
@@ -751,10 +754,18 @@ static double
 initial_seed(void)
 {
     double result;
-#if defined(HAVE_GETTIMEOFDAY)
+#if defined(HAVE_CLOCK_GETTIME)
+    struct timespec data;
+    if (clock_gettime(CLOCK_REALTIME, &data) == 0)
+	result = (data.tv_sec * 1000000000L) + data.tv_nsec;
+    else
+	result = 0.0;
+#elif defined(HAVE_GETTIMEOFDAY)
     struct timeval data;
-    gettimeofday(&data, (struct timezone *) 0);
-    result = (data.tv_sec * 1000000) + data.tv_usec;
+    if (gettimeofday(&data, (struct timezone *) 0) == 0)
+	result = (data.tv_sec * 1000000) + data.tv_usec;
+    else
+	result = 0.0;
 #elif defined(WINVER) && (WINVER >= 0x501)
     union {
 	FILETIME ft;
@@ -798,6 +809,8 @@ bi_srand(CELL *sp)
 	c = *sp;
 	*sp = cseed;
 	cseed = c;
+	if (cseed.type != C_DOUBLE)
+	    cast1_to_d(&cseed);
     }
 
 #ifdef USE_SYSTEM_SRAND
@@ -1058,8 +1071,8 @@ bi_sub(CELL *sp)
     CELL *cp;			/* pointer to the replacement target */
     CELL tc;			/* build the new string here */
     CELL sc;			/* copy of the target CELL */
-    char *front, *middle, *back;	/* pieces */
-    size_t front_len, middle_len, back_len;
+    char *front, *middle;	/* pieces */
+    size_t middle_len;
 
     TRACE_FUNC("bi_sub", sp);
 
@@ -1083,9 +1096,9 @@ bi_sub(CELL *sp)
 		     0);
 
     if (middle != 0) {
-	front_len = (size_t) (middle - front);
-	back = middle + middle_len;
-	back_len = string(&sc)->len - front_len - middle_len;
+	size_t front_len = (size_t) (middle - front);
+	char *back = middle + middle_len;
+	size_t back_len = string(&sc)->len - front_len - middle_len;
 
 	if ((sp + 1)->type == C_REPLV) {
 	    STRING *sval = new_STRING0(middle_len);
@@ -1144,7 +1157,6 @@ gsub3(PTR re, CELL *repl, CELL *target)
 
     int skip0 = -1;
     size_t howmuch;
-    char *where;
 
     TRACE(("called gsub3\n"));
 
@@ -1161,11 +1173,11 @@ gsub3(PTR re, CELL *repl, CELL *target)
     output = new_STRING0(limit);
 
     for (j = 0; j <= (int) input->len; ++j) {
-	where = REmatch(input->str + j,
-			input->len - (size_t) j,
-			cast_to_re(re),
-			&howmuch,
-			(j != 0));
+	char *where = REmatch(input->str + j,
+			      input->len - (size_t) j,
+			      cast_to_re(re),
+			      &howmuch,
+			      (j != 0));
 	/*
 	 * REmatch returns a non-null pointer if it found a match.  But
 	 * that can be an empty string, e.g., for "*" or "?".  The length
