@@ -1,6 +1,6 @@
 /********************************************
 execute.c
-copyright 2008-2017,2018, Thomas E. Dickey
+copyright 2008-2020,2023, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,20 +11,20 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: execute.c,v 1.44 2018/11/15 01:20:01 tom Exp $
+ * $MawkId: execute.c,v 1.48 2023/07/25 22:25:27 tom Exp $
  */
 
-#include "mawk.h"
-#include "files.h"
-#include "code.h"
-#include "memory.h"
-#include "symtype.h"
-#include "field.h"
-#include "bi_funct.h"
-#include "bi_vars.h"
-#include "regexp.h"
-#include "repl.h"
-#include "fin.h"
+#include <mawk.h>
+#include <files.h>
+#include <code.h>
+#include <memory.h>
+#include <symtype.h>
+#include <field.h>
+#include <bi_funct.h>
+#include <bi_vars.h>
+#include <regexp.h>
+#include <repl.h>
+#include <fin.h>
 
 #include <math.h>
 
@@ -99,9 +99,7 @@ clear_aloop_stack(ALOOP_STATE * top)
     } while (top);
 }
 
-static INST *restart_label;	/* control flow labels */
-INST *next_label;
-static CELL tc;			/*useful temp */
+INST *next_label;		/* control flow label */
 
 void
 execute(INST * cdp,		/* code ptr, start execution here */
@@ -109,10 +107,14 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	CELL *fp)		/* frame ptr into eval_stack for
 				   user defined functions */
 {
+    static INST *restart_label;	/* control flow label */
+    static CELL tc;		/*useful temp */
+    static CELL missing;	/* no value (use zero) */
+
     /* some useful temporaries */
     CELL *cp;
     int t;
-    unsigned tu;
+    UInt tu;
 
     /* save state for array loops via a stack */
     ALOOP_STATE *aloop_state = (ALOOP_STATE *) 0;
@@ -350,6 +352,69 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    }
 	    break;
 
+	case A_LENGTH:
+	    /* parameter for length() was ST_NONE; improve it here */
+	    {
+		SYMTAB *stp = (SYMTAB *) cdp->ptr;
+		cdp--;
+		TRACE(("patching %s\n", type_to_str(stp->type)));
+		switch (stp->type) {
+		case ST_VAR:
+		    cdp[0].op = _PUSHI;
+		    cdp[1].ptr = stp->stval.cp;
+		    break;
+		case ST_ARRAY:
+		    cdp[0].op = A_PUSHA;
+		    cdp[1].ptr = stp->stval.array;
+		    assert(cdp[2].op == _BUILTIN);
+		    cdp[3].ptr = (PTR) bi_alength;
+		    break;
+		case ST_NONE:
+		    cdp[0].op = _PUSHI;
+		    cdp[1].ptr = &missing;
+		    break;
+		default:
+		    bozo("execute A_LENGTH");
+		    /* NOTREACHED */
+		}
+	    }
+	    /* resume, interpreting the updated code */
+	    break;
+
+	case _LENGTH:
+	    /* parameter for length() was ST_LOCAL_NONE; improve it here */
+	    {
+		DEFER_LEN *dl = (DEFER_LEN *) cdp->ptr;
+		FBLOCK *fbp = dl->fbp;
+		short offset = dl->offset;
+		int type = fbp->typev[offset];
+
+		ZFREE(dl);
+		cdp--;
+		TRACE(("patching %s\n", type_to_str(type)));
+		switch (type) {
+		case ST_LOCAL_VAR:
+		    cdp[0].op = L_PUSHI;
+		    cdp[1].op = offset;
+		    break;
+		case ST_LOCAL_ARRAY:
+		    cdp[0].op = LA_PUSHA;
+		    cdp[1].op = offset;
+		    assert(cdp[2].op == _BUILTIN);
+		    cdp[3].ptr = (PTR) bi_alength;
+		    break;
+		case ST_LOCAL_NONE:
+		    cdp[0].op = _PUSHI;
+		    cdp[1].ptr = &missing;
+		    break;
+		default:
+		    bozo("execute _LENGTH");
+		    /* NOTREACHED */
+		}
+	    }
+	    /* resume, interpreting the updated code */
+	    break;
+
 	case SET_ALOOP:
 	    {
 		ALOOP_STATE *ap = ZMALLOC(ALOOP_STATE);
@@ -537,8 +602,6 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    sp->dval = cp->dval;
 	    break;
 
-	    /* will anyone ever use these ? */
-
 	case F_ADD_ASG:
 	    if (sp->type != C_DOUBLE)
 		cast1_to_d(sp);
@@ -1028,12 +1091,6 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    sp->dval = t ? 1.0 : 0.0;
 	    break;
 
-	case A_LENGTH:
-	    dec_sp();
-	    sp->type = C_DOUBLE;
-	    sp->dval = (double) (((ARRAY) ((sp + 0)->ptr))->size);
-	    break;
-
 	case A_TEST:
 	    /* entry :  sp[0].ptr-> an array
 	       sp[-1]  is an expression
@@ -1476,7 +1533,7 @@ DB_cell_destroy(CELL *cp)
 static UInt
 d_to_index(double d)
 {
-    if (d >= Max_Int) {
+    if (d >= (double) Max_Int) {
 	return (UInt) Max_Int;
     } else if (d >= 0.0) {
 	return (UInt) (Int) (d);
